@@ -23,13 +23,6 @@ from character import Character
 import animatorystates
 from location import Location
 
-#: The speed of the bike.
-SPEED = 3
-#: The momentum increment when the bike is pedalled.
-PEDAL_MOMENTUM = 2
-#: The maximum momentum of the bike.
-MAX_MOMENTUM = 50
-
 class Bike(Character):
     """The bike.
 
@@ -38,8 +31,12 @@ class Bike(Character):
     :param initial_as: The bike's initial animatory state.
     :param unchain_location: The location at which the bike will be deposited
                              when it's unchained from the tree.
+    :param move_delay: The delay between consecutive movements of the bike when
+                       wheeling along or being pedalled.
+    :param pedal_momentum: The momentum increment when the bike is pedalled.
+    :param max_momentum: The maximum momentum the bike can have.
     """
-    def __init__(self, bike_id, command_list_id, initial_as, unchain_location):
+    def __init__(self, bike_id, command_list_id, initial_as, unchain_location, move_delay, pedal_momentum, max_momentum):
         Character.__init__(self, bike_id, bike_id)
         self.command_list_id = command_list_id
         self.animatory_state = self.initial_as = initial_as
@@ -48,6 +45,9 @@ class Bike(Character):
         self.x, self.y = -3, 0
         self.initial_location = Location((self.x, self.y))
         self.momentum = 0
+        self.move_delay = move_delay
+        self.pedal_momentum = pedal_momentum
+        self.max_momentum = max_momentum
 
     def get_command_list_id(self, lesson_id):
         """Return the ID of the command list used by the bike.
@@ -74,7 +74,7 @@ class Bike(Character):
 
     def pedal(self):
         """Pedal the bike (increase its momentum)."""
-        self.momentum = min(self.momentum + PEDAL_MOMENTUM, MAX_MOMENTUM)
+        self.momentum = min(self.momentum + self.pedal_momentum, self.max_momentum)
 
     def start_wheeling(self, rider):
         """Start wheeling the bike along after the rider has dismounted or
@@ -83,7 +83,7 @@ class Bike(Character):
         :type rider: :class:`~character.Character`
         :param rider: The rider.
         """
-        self.walk_delay = SPEED
+        self.walk_delay = self.move_delay
         self.x, self.y, self.direction = rider.x, rider.y, rider.direction
         self.animatory_state = animatorystates.BIKE_UPRIGHT
 
@@ -91,27 +91,22 @@ class Bike(Character):
         """Continue wheeling the bike along until it hits a barrier or runs out
         of momentum.
         """
-        self.walk_delay = SPEED
+        self.walk_delay = self.move_delay
         self.momentum = max(self.momentum - 1, 0)
         if self.hit_barrier():
             self.momentum = 0
         else:
             self.x += self.direction
 
-    def hit_barrier(self, barrier_id=None):
-        """Check whether the bike has hit a barrier.
+    def hit_barrier(self):
+        """Return the wall or door that the bike has hit (if any).
 
-        :param barrier_id: If set, check whether the bike has hit the barrier
-                           with this ID; otherwise check all barriers.
         :return: The barrier that was hit, or `None` if none was hit.
         """
         # Check from a distance of 1 if travelling leftwards so that the bike
         # does not appear too close to the barrier if it hits it
         distance = 1 if self.direction < 0 else 0
-        barrier =  self.skool.barrier(self, distance)
-        if barrier_id:
-            return barrier and barrier.barrier_id == barrier_id
-        return barrier is not None
+        return self.skool.barrier(self, distance)
 
     def fall(self, rider=None):
         """Make the bike fall over (as when it has lost momentum).
