@@ -40,17 +40,17 @@ class Cast:
     """The control centre from which the entire cast of characters is
     monitored.
 
+    :type config: dict
+    :param config: Configuration parameters from the ini file.
     :type screen: :class:`~graphics.Screen`
     :param screen: The screen to draw on.
     :type gallery: :class:`~graphics.Gallery`
     :param gallery: The gallery of images to use for drawing.
-    :type config: dict
-    :param config: Configuration parameters from the ini file.
     """
-    def __init__(self, screen, gallery, config):
+    def __init__(self, config, screen, gallery):
+        self.config = config
         self.screen = screen
         self.sprites = gallery.get_image(graphics.SPRITES)
-        self.config = config
         self._extract_config(config)
         self.skool = None
         self.sprite_groups = {}
@@ -93,7 +93,8 @@ class Cast:
         :type config: dict
         :param config: Configuration parameters from the ini file.
         """
-        self.sprite_matrix_size = config.get('SpriteMatrixSize', (16, 8))
+        self.sprite_matrix_width = config.get('SpriteMatrixWidth', 16)
+        self.sprite_width, self.sprite_height = config.get('SpriteSize', (3, 4))
         self.sprite_colorkey = config.get('SpriteKey', (0, 254, 0))
         self.lines_range = config.get('LinesRange', (1, 8))
         self.lines_ink = config.get('LinesInk', 7)
@@ -337,12 +338,11 @@ class Cast:
 
         :param state: The index of the image in the matrix.
         """
-        cols, rows = self.sprite_matrix_size
-        width = self.sprites.get_width() // cols
-        height = self.sprites.get_height() // rows
-        surface = self.sprites.subsurface((width * (state % cols), height * (state // cols)), (width, height))
-        surface.set_colorkey(self.sprite_colorkey)
-        return surface
+        sprite_x = self.sprite_width * (state % self.sprite_matrix_width)
+        sprite_y = self.sprite_height * (state // self.sprite_matrix_width)
+        image = self.sprites.subsurface((sprite_x, sprite_y), (self.sprite_width, self.sprite_height))
+        image.set_colorkey(self.sprite_colorkey)
+        return image
 
     def add_sprite(self, group_id, sprite_id, sprite_index):
         """Add a sprite (an animatory state image) to the cast.
@@ -355,7 +355,7 @@ class Cast:
             self.sprite_groups[group_id] = [{}, {}]
         sprite = self._get_animatory_state(sprite_index)
         self.sprite_groups[group_id][0][sprite_id] = sprite
-        self.sprite_groups[group_id][1][sprite_id] = pygame.transform.flip(sprite, True, False)
+        self.sprite_groups[group_id][1][sprite_id] = sprite.flip()
 
     def add_eric(self, character_id, name, sprite_group_id, initial_as, direction, head_xy, flags, bend_over_hand_xy):
         """Add Eric to the cast.
@@ -944,8 +944,27 @@ class Cast:
         return [thing.get_image() for thing in self.everything]
 
     def get_speech_bubbles(self):
-        """Return a list of speech bubbles for every member of the cast."""
-        return [character.get_speech_bubble() for character in self.character_list]
+        """Return a list of speech bubbles currently in use by the cast."""
+        speech_bubbles = []
+        for character in self.character_list:
+            if character.bubble:
+                speech_bubbles.append(character.bubble)
+        return speech_bubbles
+
+    def restore(self):
+        """Perform tasks required immediately after restoring a saved game. The
+        tasks are:
+
+        * build the sprite images
+        * print the inventory
+        * print the mouse inventory
+        """
+        self.sprites.build()
+        for m in self.movables:
+            m.build_images()
+        self.eric.build_images()
+        self.eric.print_inventory()
+        self.eric.print_mouse_inventory()
 
     #//////////////////////////////////////////////////////////////////////////
     # Eric
