@@ -22,105 +22,30 @@ animated and how they behave.
 
 import sys
 import random
+import re
 from location import Location
 import ai
 from animatorystates import ARM_UP, CATAPULT0, CATAPULT1, HITTING0, HITTING1, KISSING_ERIC, KNOCKED_OUT, KNOCKED_OVER, RIDING_BIKE0, RIDING_BIKE1, SITTING_ON_CHAIR, SITTING_ON_FLOOR, WALK0, WALK1, WALK2, WALK3, WATERPISTOL
 import lines
 import items
-from lesson import Lesson, QAGenerator
+from lesson import QAGenerator
 import sound
 import debug
 
-#: Valid safe combination characters.
-SAFE_SECRETS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-#: Valid bike combination characters.
-BIKE_SECRETS = '0123456789'
-#: Valid storeroom combination characters.
-STOREROOM_SECRETS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-#: Ink colour of the secret revelation message box.
-SECRET_INK = 7
-#: Paper colour of the secret revelation message box.
-SECRET_PAPER = 0
-
-#: Lines message template.
-LINES_MSG = '%i0 LINES'
-#: Min and max number of lines (divided by 100) that may be given.
-LINES_RANGE = (1, 8)
-#: Paper colour of the lines message box when Eric is the recipient.
-LINES_PAPER_ERIC = 2
-#: Paper colour of the lines message box when anyone else is the recipient.
-LINES_PAPER_OTHER = 4
-#: Ink colour of the lines message box.
-LINES_INK = 7
-
-#: Condition indicating a dirty blackboard.
-BOARD_DIRTY = 'BoardDirty'
-
-#: Initial value of a character's kiss counter.
-KISSES = 40
-#: How much the kiss counter is decremented by each kiss.
-KISS_DECREMENT = 7
-
-#: Maximum distance at which a stinkbomb can be smelt.
-STINK_RANGE = 3
-
-#: Frequency at which a slow-moving character is moved.
-SLOW = 5
-#: Frequency at which a fast-moving character is moved.
-FAST = 3
-#: Frequency at which an even faster-moving character is moved.
-FASTER = 2
-
-#: Maximum distance at which a musophobe can detect a mouse.
-MOUSE_PROXIMITY = 5
-
-#: Min and max values of the delay between a character's speed changes.
-SPEED_CHANGE_DELAY_RANGE = (16, 31)
-
-#: The height of a character's sprite.
-SPRITE_HEIGHT = 4
-
-#: What the skool clock is set to when a character is paralysed by a falling
-#: conker.
-CONKER_CLOCK_TICKS = 1200
-#: The time at which a paralysed character remobilises.
-CONKER_WAKE_TIME = 200
-
-#: ID of water (as fired from a water pistol).
-WATER = 'WATER'
-#: ID of sherry (as fired from a water pistol).
-SHERRY = 'SHERRY'
-
-#: Min and max page numbers to substitute into a lesson message.
-PAGE_NUMBER_RANGE = (100, 999)
-
-#: The x-coordinate at or beyond which Eric is considered to be trying to
-#: escape from skool.
-ESCAPE_MIN_X = 96
-#: The minimum and maximum distance from the watcher that Eric must be to make
-#: the watcher raise the alarm.
-DANGER_ZONE = (2, 6)
-#: Ink colour of the alarm message box.
-ALARM_INK = 2
-#: Paper colour of the alarm message box.
-ALARM_PAPER = 6
-
-#: The x-coordinate at or beyond which Eric must be to make a monitor chase him
-#: away.
-CHASE_MIN_X = 160
-
 class Character:
     """Base class for anything in the game that moves.
 
     :param character_id: Unique ID.
     :param name: Display name.
+    :param head_xy: The coordinates of the character's head within his sprite
+                    when he's standing upright (used for collision detection).
     :param flags: Character flags.
     """
-    def __init__(self, character_id, name, flags=''):
+    def __init__(self, character_id, name, head_xy=None, flags=''):
         self.character_id = character_id
         self.name = name
+        self.head_xy = head_xy
         self.flags = flags
-        self.command_list = ai.CommandList()
         self.x, self.y = -3, 0
         self.random_locations = ()
         self.direction = 1 # 1 = facing right, -1 = facing left
@@ -131,14 +56,14 @@ class Character:
         self.fixed_speed = False
         self.speed_change_delay = 1
         self.always_runs = False
-        self.walk_delay = random.randint(FAST, SLOW)
+        self.walk_delay = None
         self.action_delay = 1
         self.staircase = None
         self.barrier = None
         self.room = None
         self.floor = None
         self.command_lists = {}
-        self.sit_down_message = None
+        self.sit_down_messages = []
         self.bubble = None
         self.blackboard_messages = []
         self.qa_generator = None
@@ -147,6 +72,7 @@ class Character:
         self.water = None
         self.stinkbomb = None
         self.lines_message = None
+        self.message_box_coords = None
         self.lines_messages = {}
         self.come_along_messages = []
         self.come_along_index = 0
@@ -156,9 +82,17 @@ class Character:
         self.safe_secret = None
         self.bike_secret = None
         self.storeroom_secret = None
-        self.kisses = KISSES
+        self.kisses = 0
         self.barrier_id = character_id
         self.stop_eric = False
+        self.width = None
+        self.height = None
+        self.command_list = ai.CommandList(self)
+
+    def __getstate__(self):
+        d = self.__dict__.copy()
+        d['bubble'] = None
+        return d
 
     def is_home(self, x):
         """Return whether the character is on the 'home' side of a given
@@ -202,8 +136,8 @@ class Character:
         """
         if not (self.is_scared_of_mice() and self.is_interruptible() and self.bubble is None):
             return
-        if self.y == mouse.y and 0 <= (mouse.x - self.x) * self.direction <= MOUSE_PROXIMITY:
-            self.add_command(ai.EvadeMouse())
+        if self.y == mouse.y and 0 <= (mouse.x - self.x) * self.direction <= self.cast.mouse_proximity:
+            self.add_command(ai.EvadeMouse(self.cast.evade_mouse_delay))
 
     def get_random_destination(self):
         """Return a location randomly chosen from the character's collection.
@@ -227,7 +161,7 @@ class Character:
         :param bottom_y: The y-coordinate of the bottom of the object.
         :param top_y: The y-coordinate of the top of the object.
         """
-        return top_y - SPRITE_HEIGHT < self.y <= bottom_y
+        return top_y - self.height < self.y <= bottom_y
 
     def can_open_door(self, door):
         """Return whether the character can open a door.
@@ -255,7 +189,7 @@ class Character:
         """Knock the character out of his seat."""
         self.sit_on_floor()
         self.add_command(ai.FindSeat(False))
-        self.add_command(ai.Dethroned())
+        self.add_command(ai.Floored(self.cast.dethroned_delay))
 
     def keep_seat(self):
         """Indicate that the character should return to the same seat if he's
@@ -269,12 +203,20 @@ class Character:
         """Return whether the character is in an animatory state and location
         that are amenable to him being knocked over.
         """
-        if self.get_floor() is None:
-            return
+        if not self.skool.on_floor(self):
+            return False
         if self.is_adult():
             return self.is_standing() or self.has_arm_raised()
         return self.is_standing() or self.is_sitting_on_chair()
 
+    def can_give_lines_now(self):
+        """Return whether the character is in an animatory state and location
+        that are amenable to him giving lines.
+        """
+        if self.skool.on_floor(self) or self.skool.on_staircase(self):
+            return self.is_standing() or self.has_arm_raised()
+        return False
+
     def is_door(self):
         """Return whether the character is a door.
 
@@ -361,31 +303,32 @@ class Character:
             bubble_y = self.y - (3 if self.is_adult() or self.is_standing() else 2)
             self.bubble = [bubble_x, bubble_y, None]
         lip_pos = (self.x + 1) % 8
-        bubble_img, done = self.screen.get_bubble(words, lip_pos, shift)
-        self.bubble[2] = bubble_img
+        self.bubble[2], done = self.screen.get_bubble(words, lip_pos, shift)
         return done
 
     def remove_bubble(self):
         """Remove the character's speech bubble."""
         self.bubble = None
 
-    def get_blackboard_edge(self):
-        """Return the x-coordinate to which this character should proceed to
-        reach the edge of the blackboard that is closer to him.
+    def get_blackboard(self):
+        """Return the blackboard in the room in which the character is
+        located, or `None` if there is none.
         """
-        if self.room and self.room.blackboard:
-            if self.x < self.room.blackboard.x - 2:
-                return self.room.blackboard.x - 2
-            return self.room.blackboard.right_x
+        if self.room:
+            return self.room.blackboard
 
-    def wipe_board(self, column, clear):
+    def wipe_board(self, column):
         """Wipe a bit of a blackboard clean.
 
         :param column: The column of the blackboard to wipe.
-        :param clear: `True` if the blackboard will be clean after wiping this
-                      column, `False` otherwise.
         """
-        self.skool.wipe_board(self.room.blackboard, column, clear)
+        self.skool.wipe_board(self.get_blackboard(), column)
+
+    def get_blackboard_backtrack(self):
+        """Return the distance a teacher should walk back after wiping a
+        blackboard.
+        """
+        return self.cast.blackboard_backtrack
 
     def write_on_board(self, message, index):
         """Make this character start or continue writing on a blackboard.
@@ -394,7 +337,13 @@ class Character:
         :param index: The index of the next character in the message to write.
         """
         self.action = 2
-        return self.skool.write_on_board(self, self.room.blackboard, message, index)
+        return self.skool.write_on_board(self, self.get_blackboard(), message, index)
+
+    def get_blackboard_pace_distance(self):
+        """Return the distance a teacher should pace up and down in front of
+        the blackboard during a lesson without a question-and-answer session.
+        """
+        return self.cast.bb_pace_distance
 
     def resolve_location_id(self, location_id):
         """Return the location with a given ID.
@@ -421,23 +370,33 @@ class Character:
                          Albert is struck by a falling conker).
         """
         self.knock_over()
-        self.kisses = max(0, self.kisses - 1)
+        self.kisses = max(0, self.kisses - self.cast.kiss_deckrement)
         if self.is_adult():
-            self.add_command(ai.KnockedOver(paralyse))
+            self.add_command(ai.KnockedOver(self.cast.knocked_over_delay, self.cast.reprimand_delay, paralyse))
             if paralyse:
                 self.skool.play_sound(sound.CONKER)
-                self.skool.rewind_clock(CONKER_CLOCK_TICKS)
+                self.skool.rewind_clock(self.cast.conker_clock_ticks)
         else:
             if self.previous_as == SITTING_ON_CHAIR:
                 self.chair().vacate()
                 self.add_command(ai.FindSeat(False, False))
-            self.add_command(ai.KnockedOut())
+            self.add_command(ai.Floored(self.cast.ko_delay))
 
     def is_time_to_wake(self):
         """Return whether it is time for the character to regain consciousness
         after being struck by a falling conker.
         """
-        return self.skool.is_time_remaining(CONKER_WAKE_TIME)
+        return self.skool.is_time_remaining(self.cast.conker_wake_time)
+
+    def will_write_on_board(self):
+        """Return whether the character will write on a blackboard. The answer
+        is used by teachers who are conducting a class without Eric.
+        """
+        return random.random() < self.cast.write_on_board_probability
+
+    def will_hit(self):
+        """Return whether the character wants to throw a punch now."""
+        return random.random() < self.cast.hit_probability
 
     def can_fire_catapult(self):
         """Return whether the character can fire a catapult. The answer will be
@@ -451,6 +410,10 @@ class Character:
         if CATAPULT0 in self.as_dict_L and CATAPULT1 in self.as_dict_L:
             return self.pellet and not self.pellet.is_visible()
 
+    def will_fire_catapult(self):
+        """Return whether the character wants to fire a catapult now."""
+        return random.random() < self.cast.fire_catapult_probability
+
     def fire_catapult(self):
         """Make the character launch his catapult pellet."""
         self.pellet.launch(self.x + self.direction, self.y, self.direction)
@@ -465,13 +428,13 @@ class Character:
         """
         return WATERPISTOL in self.as_dict_L and self.water and not self.water.is_visible()
 
-    def fire_water_pistol(self, liquid=WATER):
+    def fire_water_pistol(self, liquid=None):
         """Make the character fire his water pistol. A stream of water or
         sherry will begin its trajectory as a result.
 
-        :param liquid: :data:`WATER` or :data:`SHERRY`.
+        :param liquid: The ID of the liquid.
         """
-        self.water.launch(self.x + 2 * self.direction, self.y - 2, self.direction, liquid)
+        self.water.launch(self.x + 2 * self.direction, self.y - 2, self.direction, liquid or self.cast.water_id)
 
     def can_drop_stinkbomb(self):
         """Return whether the character can drop a stinkbomb. The answer will
@@ -500,7 +463,8 @@ class Character:
 
     def stalk(self, character_id):
         """Set the character's destination equal to that of another character's
-        destination (if they are both under the control of a :class:`~ai.GoTo` command).
+        destination (if they are both under the control of a :class:`~ai.GoTo`
+        command).
         """
         self.command_list.set_GoTo_destination(self.get_destination(character_id))
 
@@ -519,42 +483,63 @@ class Character:
             if blackboard:
                 return blackboard.shows(self.special_answer)
 
+    def _print_secret(self, secret):
+        """Print a message box containing a safe, bike or storeroom combination
+        character.
+
+        :param secret: The combination character.
+        """
+        self.print_message_box((secret,), self.cast.secret_ink, self.cast.secret_paper, sound.LINES2)
+
     def reveal_safe_secret(self, decked):
         """Make the character reveal his safe combination letter if certain
         conditions are met. The conditions are:
 
-        * the character holds a safe combination letter
-        * all the shields are flashing
         * the character is on-screen
-        * the character has been knocked over, or can see his special answer
-          on a blackboard
+        * the character holds a safe combination letter
+        * the safe has not been opened yet
+        * the character can see his special answer on a blackboard, or the
+          character has no special answer and all the shields are flashing
 
         :param decked: Whether the character has been knocked over.
         """
-        if not self.safe_secret or not self.skool.can_reveal_safe_secret():
-            return
         if not self.screen.contains(self):
             return
-        if decked:
-            reveal = self.special_answer is None
-        else:
-            reveal = self.can_see_special_answer()
+        flashing = self.special_answer is None
+        if not self.safe_secret or not self.skool.can_reveal_safe_secret(flashing):
+            return
+        reveal = flashing if decked else self.can_see_special_answer()
         if reveal:
-            self.print_lines_bubble(('', self.safe_secret), SECRET_INK, SECRET_PAPER, 1)
+            self._print_secret(self.safe_secret)
 
     def reveal_bike_secret(self):
         """Make the character reveal his bike combination digit (if he has
         one, and he is on-screen).
         """
         if self.bike_secret and self.screen.contains(self):
-            self.print_lines_bubble(('', self.bike_secret), SECRET_INK, SECRET_PAPER, 1)
+            self._print_secret(self.bike_secret)
 
     def reveal_storeroom_secret(self):
         """Make the character reveal his storeroom combination letter (if he
         has one, and he is on-screen).
         """
         if self.storeroom_secret and self.screen.contains(self):
-            self.print_lines_bubble(('', self.storeroom_secret), SECRET_INK, SECRET_PAPER, 1)
+            self._print_secret(self.storeroom_secret)
+
+    def _get_head_coords(self):
+        """Return the coordinates of the character's head."""
+        head_x, head_y = self.x + self.head_xy[0], self.y + self.head_xy[1]
+        if self.is_adult() and self.is_knocked_over():
+            head_y += 1
+        return head_x, head_y
+
+    def head_at(self, x, y):
+        """Return whether the character's head is at a given location.
+
+        :param x: The x-coordinate of the location.
+        :param y: The x-coordinate of the location.
+        """
+        return (x, y) == self._get_head_coords()
 
     def check_shields_at(self, x, y):
         """Return whether there is a shield at a given location.
@@ -615,7 +600,7 @@ class Character:
         :param stinkbomb: The stinkbomb to check.
         """
         if self.can_smell_stinkbombs() and self.is_standing() and self.y == stinkbomb.y:
-            return abs(stinkbomb.x - self.x) <= STINK_RANGE
+            return abs(stinkbomb.x - self.x) <= stinkbomb.stink_range
         return False
 
     def open_window(self, window):
@@ -635,7 +620,7 @@ class Character:
     #//////////////////////////////////////////////////////////////////////////
     # Character construction
     #//////////////////////////////////////////////////////////////////////////
-    def set_components(self, cast, skool, screen):
+    def set_components(self, cast, skool, screen, config):
         """Set the character up with access to essential objects.
 
         :type cast: :class:`~cast.Cast`
@@ -644,8 +629,18 @@ class Character:
         :param skool: The skool.
         :type screen: :class:`~graphics.Screen`
         :param screen: The screen.
+        :type config: dict
+        :param config: Configuration parameters from the ini file.
         """
         self.cast = cast
+        self.kisses = self.cast.initial_kiss_count
+        self.slow_delay = config.get('GoSlow', 5)
+        self.fast_delay = config.get('GoFast', 3)
+        self.faster_delay = config.get('GoFaster', 2)
+        self.walk_delay = random.randint(self.fast_delay, self.slow_delay)
+        self.speed_change_delay_range = config.get('SpeedChangeDelayRange', (16, 31))
+        self.width = self.cast.sprite_width
+        self.height = self.cast.sprite_height
         self.skool = skool
         self.screen = screen
 
@@ -659,6 +654,13 @@ class Character:
         self.x = x
         self.y = y
 
+    def add_sit_down_message(self, message):
+        """Add a sit-down message to the character's repertoire.
+
+        :param message: The message to add.
+        """
+        self.sit_down_messages.append(message)
+
     def add_blackboard_message(self, message):
         """Add a blackboard message to the character's repertoire.
 
@@ -713,14 +715,14 @@ class Character:
         """
         self.lesson_messages.append((message, condition))
 
-    def add_command_list(self, lesson_id, tap_id):
+    def add_command_list(self, lesson_id, command_list_id):
         """Add a command list ID to the character's personal timetable.
 
         :param lesson_id: The ID of the lesson in which the command list will
                           be used.
-        :param tap_id: The command list ID.
+        :param command_list_id: The command list ID.
         """
-        self.command_lists[lesson_id] = tap_id
+        self.command_lists[lesson_id] = command_list_id
 
     #//////////////////////////////////////////////////////////////////////////
     # Initialisation
@@ -733,16 +735,14 @@ class Character:
         if self.qa_generator:
             self.special_answer = self.qa_generator.initialise_special_answer()
 
-    def initialise_animatory_state(self, initial_as):
-        """Initialise the character's animatory state. The value of
-        `initial_as` determines the direction the character will face and which
-        one of his four walking animatory states he will assume.
+    def initialise_animatory_state(self, initial_as, direction):
+        """Set the character's initial animatory state and direction.
 
-        :param initial_as: One of: `L0`, `L1`, `L2`, `L3`, `R0`, `R1`, `R2`,
-                           `R3`.
+        :param initial_as: The animatory state.
+        :param direction: The direction.
         """
-        self.direction = self.initial_direction = -1 if initial_as[0].upper() == 'L' else 1
-        self.animatory_state = self.initial_as = self.walk_states[int(initial_as[1:])]
+        self.direction = self.initial_direction = direction
+        self.animatory_state = self.initial_as = initial_as
 
     def reinitialise(self):
         """Reinitialise the character after a game has ended or restarted. In
@@ -762,16 +762,16 @@ class Character:
         self.speed = 1
         self.fixed_speed = False
         self.speed_change_delay = 1
-        self.command_list = ai.CommandList()
-        self.kisses = KISSES
+        self.kisses = self.cast.initial_kiss_count
         self.stop_eric = False
+        self.command_list = ai.CommandList(self)
 
     def initialise_safe_secret(self):
         """Set and return a safe combination letter (chosen at random) for the
         character to hold.
         """
         if self.has_safe_secret():
-            self.safe_secret = random.choice(SAFE_SECRETS)
+            self.safe_secret = random.choice(self.cast.safe_secrets)
         return self.safe_secret
 
     def initialise_bike_secret(self):
@@ -779,7 +779,7 @@ class Character:
         character to hold.
         """
         if self.has_bike_secret():
-            self.bike_secret = random.choice(BIKE_SECRETS)
+            self.bike_secret = random.choice(self.cast.bike_secrets)
         return self.bike_secret
 
     def initialise_storeroom_secret(self):
@@ -787,7 +787,7 @@ class Character:
         the character to hold.
         """
         if self.has_storeroom_secret():
-            self.storeroom_secret = random.choice(STOREROOM_SECRETS)
+            self.storeroom_secret = random.choice(self.cast.storeroom_secrets)
         return self.storeroom_secret
 
     #//////////////////////////////////////////////////////////////////////////
@@ -972,7 +972,9 @@ class Character:
 
         :param character_id: The ID of the character to get the destination of.
         """
-        return self.cast.get(character_id).get_GoTo_destination()
+        character = self.cast.get(character_id)
+        if character:
+            return character.get_GoTo_destination()
 
     def has_line_of_sight_to(self, character):
         """Return whether the character has a line of sight to another
@@ -1010,23 +1012,11 @@ class Character:
         """Return whether the character can open doors and windows."""
         return 'D' in self.flags
 
-    def can_expel_eric(self):
-        """Return whether the character is responsible for expelling Eric when
-        he has exceeded the lines limit.
-        """
-        return 'E' in self.flags
-
     def is_punchable(self):
         """Return whether the character can be knocked over by a punch.
         """
         return 'F' in self.flags
 
-    def can_shadow_eric(self):
-        """Return whether the character is responsible for shadowing Eric after
-        being alerted that Eric is escaping.
-        """
-        return 'H' in self.flags
-
     def has_safe_key(self):
         """Return whether the character holds the key to the safe."""
         return 'K' in self.flags
@@ -1059,6 +1049,23 @@ class Character:
         """Return whether the character holds a safe combination letter."""
         return 'S' in self.flags
 
+    def is_trippable(self):
+        """Return whether the character can be tripped up by a stampeding kid.
+        """
+        return 'T' in self.flags
+
+    def adds_to_lines_total(self):
+        """Return whether any lines the character gets are added to Eric's
+        total.
+        """
+        return 'U' in self.flags
+
+    def adds_to_score(self):
+        """Return whether any lines the character gets are added to Eric's
+        score.
+        """
+        return 'V' in self.flags
+
     def sometimes_runs(self):
         """Return whether the character sometimes runs instead of walking (as
         the kids do).
@@ -1125,7 +1132,7 @@ class Character:
 
         :param command_list_id: The ID of the command list to switch to.
         """
-        self.cast.change_command_list(self, command_list_id)
+        self.cast.change_command_list(self.character_id, command_list_id)
 
     def restart_command_list(self):
         """Restart the character's command list."""
@@ -1180,13 +1187,14 @@ class Character:
         as_dict = self.as_dict_R if self.direction > 0 else self.as_dict_L
         return (self.x, self.y, as_dict[self.animatory_state])
 
-    def get_speech_bubble(self):
-        """Return the character's speech bubble. The speech bubble is
-        represented as a three-element tuple, `(x, y, img)`, where `x` and `y`
-        are the coordinates of the bubble, and `img` is the image (or `None` if
-        the character is not speaking).
+    def build_images(self):
+        """Build the animatory state images for the character. This method is
+        called after rescaling the screen or loading a saved game.
         """
-        return self.bubble or (0, 0, None)
+        for image in self.as_dict_L.values():
+            image.build()
+        for image in self.as_dict_R.values():
+            image.build()
 
     #//////////////////////////////////////////////////////////////////////////
     # Eric
@@ -1229,7 +1237,7 @@ class Character:
         self.animatory_state = KISSING_ERIC
         self.x += self.direction
         self.direction *= -1
-        self.kisses = max(self.kisses - KISS_DECREMENT, 0)
+        self.kisses = max(self.kisses - self.cast.kiss_decrement, 0)
 
     def is_kissing_eric(self):
         """Return whether the character is kissing Eric."""
@@ -1278,6 +1286,12 @@ class Character:
         """Unfreeze Eric (see :meth:`freeze_eric`)."""
         self.cast.unfreeze_eric()
 
+    def get_tell_eric_delay(self):
+        """Return the time to wait for Eric to respond before repeating a
+        message.
+        """
+        return self.cast.tell_eric_delay
+
     def eric_understood(self):
         """Return whether Eric has acknowledged understanding of a message
         delivered to him.
@@ -1293,43 +1307,53 @@ class Character:
 
         :param lines: The number of lines to add.
         """
-        self.skool.add_lines(lines // 10)
+        self.skool.add_lines(lines)
 
-    def should_stop_eric(self):
+    def should_stop_eric(self, escape_x, danger_zone):
         """Return whether this character should try to prevent Eric from
         escaping.
+
+        :param escape_x: The x-coordinate beyond which Eric should be regarded
+                         as trying to escape.
+        :param danger_zone: The minimum and maximum distance to the left of the
+                            watcher that Eric must be for him to raise the
+                            alarm.
         """
         eric_x, eric_y = self.cast.get_location_of_eric()
         if eric_y != self.y or self.direction > 0:
             return False
-        return eric_x >= ESCAPE_MIN_X and self.x - DANGER_ZONE[1] <= eric_x <= self.x - DANGER_ZONE[0]
+        return eric_x >= escape_x and self.x - danger_zone[1] <= eric_x <= self.x - danger_zone[0]
 
-    def raise_alarm(self, message, command_list_id):
+    def raise_alarm(self, message, alertee_id, command_list_id):
         """Make this character raise the alarm that Eric is escaping.
 
         :param message: The alarm message.
+        :param alertee_id: The ID of the character to alert.
         :param command_list_id: The ID of the command list that the alerted
                                 character should switch to.
         """
-        self.print_lines_bubble(self.skool.expand_message(message), ALARM_INK, ALARM_PAPER, 1)
+        self.print_message_box(self.skool.expand_message(message), self.cast.escape_alarm_ink, self.cast.escape_alarm_paper, sound.LINES2)
         if not self.cast.is_eric_expelled():
-            self.cast.shadow_eric(command_list_id)
+            self.cast.change_command_list(alertee_id, command_list_id)
 
     def is_stopping_eric(self, eric):
         """Return whether this character is preventing Eric from escaping."""
         return self.stop_eric and eric.impeded(self.y + 3, self.y) and self.direction != eric.direction and abs(self.x - eric.x) == 2
 
-    def should_chase_eric(self):
+    def should_chase_eric(self, chase_x):
         """Return whether the character should start chasing Eric away. The
         answer will be `True` if all of the following conditions are met:
 
         * it is not playtime
-        * Eric is in the girls' skool
+        * Eric's x-coordinate is greater than `chase_x`
         * Eric is on the same floor as the character
+
+        :param chase_x: The x-coordinate beyond which Eric must be for the
+                        monitor to start chasing Eric.
         """
         if not self.skool.is_playtime():
             eric_x, eric_y = self.cast.get_location_of_eric()
-            return eric_x >= CHASE_MIN_X and eric_y == self.y
+            return eric_x >= chase_x and eric_y == self.y
         return False
 
     #//////////////////////////////////////////////////////////////////////////
@@ -1337,17 +1361,20 @@ class Character:
     #//////////////////////////////////////////////////////////////////////////
     def start_lesson(self):
         """Called by the swot to start a lesson."""
-        self.lesson = Lesson(self.cast, self, self.room)
+        self.lesson = self.cast.create_lesson(self, self.room)
         self.skool.lesson = self.lesson
 
-    def join_lesson(self, with_qa):
+    def join_lesson(self, qa_group):
         """Called by the teacher to join the lesson started by the swot.
 
-        :param with_qa: If `True`, the teacher will start a
-                        question-and-answer session with the swot.
+        :param qa_group: The Q&A group from which to choose questions and
+                         answers for the teacher and the swot; if `None`, the
+                         Q&A group will be chosen at random from those
+                         available each time a question and answer is
+                         generated.
         """
         self.lesson = self.skool.lesson
-        self.lesson.join(self, self.qa_generator if with_qa else None)
+        self.lesson.join(self, self.qa_generator, qa_group)
 
     def next_swot_action(self):
         """Return the next command to be executed by the swot (which may be
@@ -1400,36 +1427,53 @@ class Character:
         if not self.screen.contains(self):
             return
         recipient = self.cast.get(recipient_id)
-        admonition = self.lines_messages.get(message_id, None)
+        admonition = self.lines_messages.get(message_id)
         if not (recipient and admonition):
             return
         if recipient.is_eric() and recipient.expelled:
             return
-        num_lines = 10 * random.randint(*LINES_RANGE)
-        message = (LINES_MSG % num_lines, recipient.name)
+        num_lines = 100 * random.randint(*self.cast.lines_range)
+        message = self.skool.get_lines_message(num_lines, recipient.name)
         if recipient.is_eric():
-            paper = LINES_PAPER_ERIC
+            paper = self.cast.lines_paper_eric
             self.skool.add_lines(num_lines)
         else:
-            paper = LINES_PAPER_OTHER
-            self.skool.add_to_score(num_lines)
-        self.print_lines_bubble(message, LINES_INK, paper, 0)
-        self.print_lines_bubble(admonition, LINES_INK, paper, 1)
+            paper = self.cast.lines_paper_other
+            if recipient.adds_to_score():
+                self.skool.add_to_score(num_lines)
+            elif recipient.adds_to_lines_total():
+                self.skool.add_lines(num_lines)
+        ink = self.cast.lines_ink
+        pre_resume = self.print_message_box
+        # It is possible for a character to change location between the
+        # printing of the first lines bubble and the printing of the second;
+        # thus we use prev_coords=True to ensure that the first and second
+        # lines bubbles are printed in the same place
+        pre_resume_args = (admonition, ink, paper, sound.LINES2, True)
+        self.print_message_box(message, ink, paper, sound.LINES1, pre_resume=pre_resume, pre_resume_args=pre_resume_args)
 
-    def print_lines_bubble(self, message, ink, paper, effect):
-        """Print a lines message box above the character's head.
+    def print_message_box(self, message, ink, paper, sound_id, prev_coords=False, pre_resume=None, pre_resume_args=None):
+        """Print a message box above the character's head.
 
         :param message: The text to display in the message box.
         :type ink: number
         :param ink: The ink colour of the message box.
         :type paper: number
         :param paper: The paper colour of the message box.
-        :type effect: number
-        :param effect: The sound effect to play (0 or 1).
+        :param sound_id: The ID of the sound effect to play.
+        :param prev_coords: If `True`, print the message box at the same
+                            coordinates as the last one; otherwise calculate
+                            the coordinates afresh.
+        :param pre_resume: A method to execute after the sound effect has
+                           finished playing.
+        :param pre_resume_args: The arguments for the `pre_resume` method.
         """
-        y = self.y - 2 if self.is_knocked_over() else self.y - 3
-        self.screen.print_lines_bubble(self.x, y, message, ink, paper)
-        self.skool.make_lines_sound(effect)
+        if not (self.message_box_coords and prev_coords):
+            self.message_box_coords = self._get_head_coords()
+        x, y = self.message_box_coords
+        self.screen.print_message_box(x, y, message, ink, paper)
+        self.skool.locked = pre_resume is not None
+        self.skool.play_sound(sound_id, pre_resume=pre_resume, pre_resume_args=pre_resume_args)
 
     def give_lines(self, recipient_id, message_id, now=False):
         """Add a lines message to the character's buffer.
@@ -1484,12 +1528,21 @@ class Character:
         """
         return self.skool.get_assembly_message()
 
+    def get_sit_down_message(self):
+        """Return a sit-down message randomly chosen from the character's
+        collection.
+        """
+        if self.sit_down_messages:
+            message = random.choice(self.sit_down_messages)
+            return self.cast.expand_names(message)
+
     def get_blackboard_message(self):
         """Return a blackboard message randomly chosen from the character's
         collection.
         """
-        message = random.choice(self.blackboard_messages)
-        return self.cast.expand_names(message)
+        if self.blackboard_messages:
+            message = random.choice(self.blackboard_messages)
+            return self.cast.expand_names(message)
 
     def get_come_along_message_id(self):
         """Return the ID of the message for Eric's teacher to use next as he
@@ -1510,7 +1563,7 @@ class Character:
         :return: `True` if the condition is true or the condition identifier
                  is unrecognised; `False` otherwise.
         """
-        if condition == BOARD_DIRTY:
+        if condition == self.cast.board_dirty:
             return self.room.blackboard_dirty()
         return True
 
@@ -1518,11 +1571,25 @@ class Character:
         """Return a lesson message randomly chosen from the character's
         collection.
         """
-        while True:
-            message, condition = random.choice(self.lesson_messages)
+        message = None
+        messages = self.lesson_messages[:]
+        while messages:
+            message, condition = messages.pop(random.randrange(len(messages)))
             if self._lesson_message_condition(condition):
+                return self._expand_page_numbers(message)
+
+    def _expand_page_numbers(self, message):
+        """Expand any page number macros in a lesson message and return the
+        result.
+        """
+        while True:
+            search = re.search('\$\( *[0-9]+, *[0-9]+ *\)', message)
+            if not search:
                 break
-        return message.replace('$n', str(random.randint(*PAGE_NUMBER_RANGE)))
+            macro = search.group()
+            page_range = eval(macro[1:])
+            message = message.replace(macro, str(random.randint(*page_range)), 1)
+        return message
 
     def expand_names(self, message):
         """Return a message with character name macros expanded. A character
@@ -1557,27 +1624,27 @@ class Character:
         """Reset the delay before the character is moved again (if he is
         walking).
         """
-        self.walk_delay = FAST if self.speed == 2 else SLOW
+        self.walk_delay = self.fast_delay if self.speed == 2 else self.slow_delay
 
     def is_time_to_move(self):
         """Return whether the character should be moved on this pass."""
         if self.bubble:
-            self.action_delay = (self.action_delay + 1) % FAST
+            self.action_delay = (self.action_delay + 1) % self.fast_delay
             return self.action_delay == 0
         if self.action in (1, 2):
             # Writing or not moving
-            self.action_delay = (self.action_delay + 1) % SLOW
+            self.action_delay = (self.action_delay + 1) % self.slow_delay
             return self.action_delay == 0
         if self.action == 3:
             # Perform hitting, firing etc. quickly
-            self.action_delay = (self.action_delay + 1) % FASTER
+            self.action_delay = (self.action_delay + 1) % self.faster_delay
             return self.action_delay == 0
         self.walk_delay -= 1
         if self.walk_delay > 0:
             return False
         self.speed_change_delay -= 1
         if self.speed_change_delay == 0:
-            self.speed_change_delay = random.randint(*SPEED_CHANGE_DELAY_RANGE)
+            self.speed_change_delay = random.randint(*self.speed_change_delay_range)
             if not self.fixed_speed:
                 if self.always_runs:
                     self.speed = 2
@@ -1607,7 +1674,7 @@ class Character:
         self.barrier = self.skool.barrier(self)
         self.room = self.skool.room(self)
         self.floor = self.get_floor()
-        self.command_list.command(self)
+        self.command_list.command()
 
     def left(self):
         """Move the character leftwards."""
